/***********************************************************
 * $Id: AnnotationHttpContextProvider.java 607 2012-01-10 07:53:29Z gwt $
 * 
 * HTTP/AOP Utilities of clazzes.org
 * http://www.clazzes.org
 *
 * Created: 09.01.2012
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.aop;

import java.text.ParsePosition;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.aopalliance.intercept.MethodInvocation;
import org.clazzes.util.http.UrlHelper;

/**
 * A HTTP context provider, which parses {@link InvocationHttpContext}
 * annotations on the given method invocation and evaluates the
 * context URL as described in the documentation of the annotation class.
 */
public class AnnotationHttpContextProvider implements HttpContextProvider {

	private static final Map<Class<?>,Integer> primitiveTypes;
	
	static {
		primitiveTypes = new HashMap<Class<?>,Integer>();
		primitiveTypes.put(Boolean.TYPE,0);
		primitiveTypes.put(Character.TYPE,1);
		primitiveTypes.put(Byte.TYPE,2);
		primitiveTypes.put(Short.TYPE,3);
		primitiveTypes.put(Integer.TYPE,4);
		primitiveTypes.put(Long.TYPE,5);
		primitiveTypes.put(Float.TYPE,6);
		primitiveTypes.put(Double.TYPE,7);
	}
	
	/**
     * @param property The property for which to retrieve the getter name.
     * @return The name of the getter method for the given property.
     * 
     * @see PropertyHelper#getBooleanGetterName(String)
     */
    private static String getGetterName(String property)
    {
        return "get" + Character.toUpperCase(property.charAt(0)) + property.substring(1);
    }

    private static Object evalMember(Object root,Object ctxt, String arg, ParsePosition pos) throws Exception {
		
		Object idx_o = evalArg(root,root,arg,pos);

		if (pos.getIndex() >= arg.length() || arg.charAt(pos.getIndex()) != ']') {
			throw new IllegalArgumentException("Illegal argument string ["+arg+"], no closing brace in array subscript.");
		}
		
		pos.setIndex(pos.getIndex()+1);

		Class<? extends Object> vc = ctxt.getClass();

		if (Map.class.isAssignableFrom(vc)) {
			return ((Map<?,?>)ctxt).get(idx_o);
		}
		 
		if (!(idx_o instanceof Number)) {
			
			throw new IllegalArgumentException("Illegal argument string ["+arg+"], index is not a number.");
		}
		
		int idx = ((Number)idx_o).intValue();
		
		if (vc.isArray()) {
			
			Integer ck = primitiveTypes.get(vc.getComponentType());
			
			if (ck == null) {
				return ((Object[])ctxt)[idx];
			}
			else {
				switch(ck) {
				case 0:
					return ((boolean[])ctxt)[idx];
				case 1:
					return ((char[])ctxt)[idx];
				case 2:
					return ((byte[])ctxt)[idx];
				case 3:
					return ((short[])ctxt)[idx];
				case 4:
					return ((int[])ctxt)[idx];
				case 5:
					return ((long[])ctxt)[idx];
				case 6:
					return ((float[])ctxt)[idx];
				case 7:
					return ((double[])ctxt)[idx];
				default:
					throw new IllegalArgumentException("Illegal argument string ["+arg+"], internal error.");
			}
			}
		}
		else if (List.class.isAssignableFrom(vc)) {
			return ((List<?>)ctxt).get(idx);
		}
		else {
			throw new IllegalArgumentException("Illegal argument string ["+arg+"], value is not an array or list.");
		}
    }
    
    private static Object evalArg(Object root,Object ctxt, String arg, ParsePosition pos) throws Exception {
		
		int npos = pos.getIndex();
		
		Object val;
		
		if (Character.isJavaIdentifierStart(arg.charAt(npos))) {

			++npos;
			
			while (npos<arg.length() && Character.isJavaIdentifierPart(arg.charAt(npos))) {
				++npos;
			}
		
			String propName = arg.substring(pos.getIndex(),npos);
			
			val = ctxt.getClass().getMethod(getGetterName(propName)).invoke(ctxt);
		}
		else if (Character.isDigit(arg.charAt(npos))) {
			
			while (npos<arg.length() && Character.isDigit(arg.charAt(++npos))) ;
			
			val = Integer.valueOf(arg.substring(pos.getIndex(),npos));
			pos.setIndex(npos);
			return val;
		}
		else {
			throw new IllegalArgumentException("Illegal argument string ["+arg+"].");
		}
		
		while (npos < arg.length()) {

			switch (arg.charAt(npos)) {
			
			case '[':
				pos.setIndex(++npos);
				val = evalMember(root,val,arg,pos);
				npos = pos.getIndex();
				break;
				
			case '.':
				pos.setIndex(++npos);
				return evalArg(root,val,arg,pos);

			default:
				pos.setIndex(npos);
				return val;
			}
		}
		
		pos.setIndex(npos);
		return val;
	}
	
	
	/* (non-Javadoc)
	 * @see org.clazzes.gwt.sec.server.HttpContextProvider#getContextUrl(org.aopalliance.intercept.MethodInvocation)
	 */
	@Override
	public String getContextUrl(MethodInvocation invocation) {
		
		InvocationHttpContext ihc =
				invocation.getMethod().getAnnotation(InvocationHttpContext.class);
		
		if (ihc == null) {
			ihc = invocation.getMethod().getDeclaringClass().getAnnotation(InvocationHttpContext.class);
		}

		if (ihc == null) {
			return null;
		}
		
		if (ihc.args() == null || ihc.args().length == 0) {
			return ihc.value();
		}
		
		Object[] args = new Object[ihc.args().length];
		
		for (int i=0;i<ihc.args().length;++i) {
		
			String arg = ihc.args()[i];
			ParsePosition pos = new ParsePosition(0);
			try {
				args[i] = evalArg(invocation,invocation,arg,pos);
				if (pos.getIndex() != arg.length()) {
					throw new IllegalArgumentException("Illegal argument string ["+arg+"].");
				}
				
				Class<? extends Object> ac = args[i].getClass();
				
				if (!Number.class.isAssignableFrom(ac) &&
						!Date.class.isAssignableFrom(ac) &&
						!Calendar.class.isAssignableFrom(ac) ) {
					
					args[i] = UrlHelper.encodeUtf8(args[i].toString());
				}
				
			} catch (Exception e) {
				throw new IllegalArgumentException("Illegal argument string ["+arg+"].");
			}
		}
		
		return String.format(Locale.US,ihc.value(),args);
	}

}
