/***********************************************************
 * $Id: HttpLoginInterceptor.java 451 2011-06-12 15:38:29Z gwt $
 * 
 * HTTP/AOP Utilities of clazzes.org
 * http://www.clazzes.org
 *
 * Created: 18.05.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.aop;

import java.security.Principal;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.clazzes.util.http.sec.HttpLoginService;
import org.clazzes.util.http.sec.LoginRequiredException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Check for a logged in user using the provided external login service and
 * an optional context provider.
 */
public class HttpLoginInterceptor extends HttpAwareSupport implements MethodInterceptor {

	private static final Logger log = LoggerFactory.getLogger(HttpLoginInterceptor.class);
		
	private HttpLoginService loginService;
	private HttpContextProvider contextProvider;

	@Override
	public Object invoke(MethodInvocation invocation) throws Throwable {
		
		Principal principal = this.loginService.checkLogin(this.getThreadLocalRequest());
		
		if (principal == null) {
			
			String url = this.loginService.getLoginUrl();
			log.warn("Try to execute operation ["+invocation.getMethod()+"] without login, throwing LoginRequiredException with login URL ["+url+"].");

			throw new LoginRequiredException(url);
		}
		
		String ctxt = null;
		
		if (this.contextProvider != null) {
			
			ctxt = this.contextProvider.getContextUrl(invocation);
		}
		
		if (ctxt != null) {
			if (!this.loginService.checkPermission(this.getThreadLocalRequest(),ctxt)) {
				
				String msg = "Permission denied on context ["+ctxt+"] to user ["+principal+"].";
				
				log.error(msg);
				throw new SecurityException(msg);
			}

			if (log.isDebugEnabled())
				log.debug("User ["+principal+"] executes operation ["+invocation.getMethod()+"] in context ["+ctxt+"].");
		}
		else {
			if (log.isDebugEnabled())
				log.debug("User ["+principal+"] executes operation ["+invocation.getMethod()+"].");
		}
		
		return invocation.proceed();
	}

	/**
	 * @return the external login service to use.
	 */
	public HttpLoginService getLoginService() {
		return loginService;
	}

	/**
	 * @param loginService the external login service to set.
	 */
	public void setLoginService(HttpLoginService loginService) {
		this.loginService = loginService;
	}
	
	/**
	 * @return the contextProvider used to map invocations to a context URL.
	 */
	public HttpContextProvider getContextProvider() {
		return contextProvider;
	}

	/**
	 * @param contextProvider the contextProvider used to map invocations to a context URL to set.
	 */
	public void setContextProvider(HttpContextProvider contextProvider) {
		this.contextProvider = contextProvider;
	}

}
