/***********************************************************
 * $Id: InvocationHttpContext.java 607 2012-01-10 07:53:29Z gwt $
 * 
 * HTTP/AOP Utilities of clazzes.org
 * http://www.clazzes.org
 *
 * Created: 09.01.2012
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.aop;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Locale;

import org.aopalliance.intercept.MethodInvocation;

/**
 * <p>An annotation, which provides a context for an invocation. It allows to replace bean
 * properties of the passed {@link MethodInvocation} to be evaluated by a reduced form of
 * bean retrieval expressions, which supports property fetching through a <code>.</code>(dot)
 * operator and subscripts in arrays.</p>
 * 
 * <p>The annotation may be attached to interfaces or individual methods as in the
 * following example:</p>
 * <pre>
 * &#64;InvocationHttpContext(value="/ctxt?m=%s&rv=%s",args={"method.name","method.returnType.simpleName"})
 * public interface RpcTestInterface extends CheckedRemoteService {
 * 	
 * 	&#64;InvocationHttpContext(value="/ctxt?id=%s",args="arguments[0]")
 * 	public void foo(String bar);
 * 	
 * 	&#64;InvocationHttpContext(value="/ctxt?id=%s",args="arguments[0].id")
 * 	public void pojoFoo(TestPOJO bar);
 * 	
 * 	&#64;InvocationHttpContext(value="/ctxt?x=%5.3f",args="arguments[0][arguments[1].count]")
 * 	public void choose(double[] arr,TestPOJO bar);
 * 	
 * 	&#64;InvocationHttpContext(value="/ctxt?nm=%s",args="arguments[0][arguments[1]].name")
 *	public void choosePojoList(List<TestPOJO> arr,int i);
 *
 * 	&#64;InvocationHttpContext(value="/ctxt?nm=%s",args="arguments[0][arguments[1]].name")
 * 	public void choosePojo(TestPOJO[] arr,int i);
 * 
 *  // use context provided by the annotation to the interface above.
 * 	public String helloWorld();
 * }
 * </pre>
 * <p>String properties are inserted to the format expression in HTML-quoted form,
 * <code>arguments</code> refers to the actual arguments passed to the method.</p>
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.METHOD,ElementType.TYPE})
public @interface InvocationHttpContext {

	/**
	 * @return The invocation context URL, which may contain
	 *         format strings processed by {@link String#format(java.util.Locale, String, Object...)}
	 *         using {@link Locale#US}. If format specifiers are present,
	 *         {@link #args()} must be set with bean expressions to set
	 *         the arguments.
	 */
	public String value();
	
	/**
	 * @return An array of bean property expressions to fill into the
	 *         format arguments of {@link #value()}. 
	 */
	public String[] args() default {};
}
